/*
Julien DeAntoni -- January, 2012
   License : WTFPL

	Example code for setting up and using timers (adapted from AvrX provided example)

 This program is free software. It comes without any warranty, to
 the extent permitted by applicable law. You can redistribute it
 and/or modify it under the terms of the WTF Public License, Version 2,
 as published by Sam Hocevar. See http://sam.zoy.org/wtfpl/COPYING for 
 more details. 
    
*/

#include "../avrx/avrx.h"
#include "hardware.h"
#include <avr/io.h>

#include <LiquidCrystal.h>



TimerControlBlock   timer1,             // Declare the control blocks needed for timers
                    timer2;
/*
 Task 1 simply flashes the light led on PORTB[0] and PORTB[2]
 */
AVRX_GCC_TASKDEF(task1, 8, 2)
{
    while (1)
    {
        AvrXStartTimer(&timer1, 800);       // 800 ms delay
        AvrXWaitTimer(&timer1);
        PORTB = PORTB ^ 0x01;

        AvrXStartTimer(&timer1, 200);       // 200 ms delay
        AvrXWaitTimer(&timer1);
	PORTB = PORTB ^ 0x01;
    }
}

LiquidCrystal* ptr_lcd;
/*
 Task 2 cycles the light PORTB[1] on/off over 2 seconds.  
 */
AVRX_GCC_TASKDEF(task2, 8, 1)
{
  static int cpt=3;
    while (1)
    {

        AvrXDelay(&timer2, 500);           // 2 second delay
	cpt+=1;
	cpt%=9;
	PORTB |=0x08;
	AvrXDelay(&timer2, 1000); 
   	ptr_lcd->setCursor(7,0); // set to the 3th column and 2nd row
   	ptr_lcd->print(cpt);
    }
}


AVRX_SIGINT(TIMER1_COMPA_vect)
{
    IntProlog();                // Switch to kernel stack/context
    TCNT1= TCNT0_INIT;
    AvrXTimerHandler();         // Call Time queue manager
    Epilog();                   // Return to tasks
}

int main(void)                 // Main runs under the AvrX Stack
{
    AvrXSetKernelStack(0);
    MCUCR = 1<<SE;
    
    //timer stuff
    // disable timer interrupts
    TIMSK1 = TCNT0_INIT;

    // cut off value = 46875 = (16*10E6 / 256) * 3seconds
    OCR1A  = 0x00FF;
    // mode - normal + prescaler 256
    TCCR1A = 0x00;
    TCCR1B = 0x0C;
    //enable compa interrupt
    TIMSK1 = 0x02;
  
    DDRB = 0xFF;

    LiquidCrystal lcd(10, 11, 12, 13, 14, 15, 16);
    ptr_lcd=&lcd;
    lcd.begin(16, 2); 
    lcd.clear();//clears the LCD and positions the cursor in the upper-left corner
   
    ptr_lcd->setCursor(0,1); // set to the 3th column and 2nd row
    ptr_lcd->print("do it yourself !");
    
    AvrXRunTask((TaskControlBlock*)TCB(task1));
    AvrXRunTask((TaskControlBlock*)TCB(task2));

    Epilog();                   // Switch from AvrX Stack to first task
    return 0;
}






extern "C" void __cxa_pure_virtual(void)
{
// call to a pure virtual function happened ... wow, should never happen ... stop
while(1)
      PORTB=0x00;
}
