/*
Julien DeAntoni -- January, 2012
   License : WTFPL

	Example code for setting up and using timers (adapted from AvrX provided example)

 This program is free software. It comes without any warranty, to
 the extent permitted by applicable law. You can redistribute it
 and/or modify it under the terms of the WTF Public License, Version 2,
 as published by Sam Hocevar. See http://sam.zoy.org/wtfpl/COPYING for 
 more details. 
    
*/

#include "../avrx/avrx.h"
#include "hardware.h"
#include <avr/io.h>

TimerControlBlock  timer1,             // Declare the control blocks needed for timers
                   timer2;
/*
 Timer 0 Overflow Interrupt Handler

 Prototypical Interrupt handler:
 . Switch to kernel context
 . handle interrupt
 . switch back to interrupted context.
 */
AVRX_SIGINT(TIMER0_OVF_vect)
{
    IntProlog();                // Switch to kernel stack/context
    TCNT0 = TCNT0_INIT;
    AvrXTimerHandler();         // Call Time queue manager
    Epilog();                   // Return to tasks
}


/*
 Task 1 simply flashes the light led on D8
 */
AVRX_GCC_TASKDEF(task1, 8, 3)
{
    while (1)
    {
        AvrXStartTimer(&timer1, 800);       // 800 ms delay
        AvrXWaitTimer(&timer1);
        PORTB = PORTB ^ 0x01;

        AvrXStartTimer(&timer1, 200);       // 200 ms delay
        AvrXWaitTimer(&timer1);
	PORTB = PORTB ^ 0x01;
    }
}
/*
 Task 2 cycles the light on D11 on/off over 2 seconds.  
 */
AVRX_GCC_TASKDEF(task2, 8, 2)
{
    while (1)
    {
        AvrXDelay(&timer2, 2000);           // 2 second delay
        PORTB = PORTB ^ 0x08;
    }
}

int main(void)                 // Main runs under the AvrX Stack
{
    AvrXSetKernelStack(0);
    
    //setting up timer0
    MCUCR = 1<<SE;
    TCNT0 = TCNT0_INIT;
    TCCR0B |= 0x04;
    TIMSK0 = 1<<TOIE0;

    //portB direction
    DDRB = 0xFF;

    AvrXRunTask(TCB(task1));
    AvrXRunTask(TCB(task2));
    
    Epilog();                   // Switch from AvrX Stack to first task
   return;
}

